<?php
    
    namespace common\models;
    
    use Yii;
    use yii\base\NotSupportedException;
    use yii\behaviors\TimestampBehavior;
    use yii\db\ActiveRecord;
    use yii\helpers\FileHelper;
    use yii\web\IdentityInterface;

    /**
     * This is the model class for table "author".
     *
     * @property int    $id
     * @property string $name
     * @property string $secondname
     * @property string $email
     * @property string $phone
     * @property string $password_hash
     * @property string $password_reset_token
     * @property string $auth_key
     * @property int    $created_at
     * @property int    $updated_at
     * @property string $about
     * @property int    $status
     */
    class Author extends ActiveRecord implements IdentityInterface
    {
        const STATUS_NOACTIVE = 1;
        const STATUS_ACTIVE = 10;
        const STATUS_BLOCKED = 0;
        /**
         * {@inheritdoc}
         */
        public static function tableName()
        {
            return 'author';
        }
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                TimestampBehavior::className(),
            ];
        }
        
        /**
         * {@inheritdoc}
         */
        public function rules()
        {
            return [
                [
                    [
                        'created_at',
                        'updated_at',
                        'status',
                    ],
                    'default',
                    'value' => null,
                ],
                [
                    [
                        'created_at',
                        'updated_at',
                        'status',
                    ],
                    'integer',
                ],
                [
                    [ 'about' ],
                    'string',
                ],
                [
                    [
                        'name',
                        'secondname',
                        'email',
                        'phone',
                        'password_hash',
                        'password_reset_token',
                        'auth_key',
                        'avatar'
                    ],
                    'string',
                    'max' => 255,
                ],
                [
                    'email', 'email'
                ],
                [
                    ['name', 'secondname', 'email'], 'required'
                ],
                [
                    'email', 'unique'
                ]
            ];
        }
        
        /**
         * {@inheritdoc}
         */
        public function attributeLabels()
        {
            return [
                'id'                   => Yii::t('app', 'ID'),
                'name'                 => Yii::t('app', 'Name'),
                'secondname'           => Yii::t('app', 'Secondname'),
                'email'                => Yii::t('app', 'Email'),
                'phone'                => Yii::t('app', 'Phone'),
                'password_hash'        => Yii::t('app', 'Password Hash'),
                'password_reset_token' => Yii::t('app', 'Password Reset Token'),
                'auth_key'             => Yii::t('app', 'Auth Key'),
                'created_at'           => Yii::t('app', 'Created At'),
                'updated_at'           => Yii::t('app', 'Updated At'),
                'about'                => Yii::t('app', 'About'),
                'status'               => Yii::t('app', 'Status'),
            ];
        }
        
        public function getStatuses(): array
        {
          return [
            self::STATUS_ACTIVE => 'Активный',
            self::STATUS_NOACTIVE => 'Неактивный',
            self::STATUS_BLOCKED => 'Заблокирован',

          ];
        }
    
        /**
         * @inheritdoc
         */
        public static function findIdentity($id)
        {
            return static::findOne(
                [
                    'id'     => $id,
                    'status' => self::STATUS_ACTIVE,
                ]
            );
        }
    
        /**
         * @inheritdoc
         */
        public static function findIdentityByAccessToken($token, $type = null)
        {
            throw new NotSupportedException('"findIdentityByAccessToken" is not implemented.');
        }
    
        /**
         * Finds user by username
         *
         * @param string $email
         *
         * @return static|null
         */
        public static function findByUsername($email)
        {
            return static::findOne(
                [
                    'email' => $email,
                    'status'   => self::STATUS_ACTIVE,
                ]
            );
        }
    
        /**
         * Finds user by password reset token
         *
         * @param string $token password reset token
         *
         * @return static|null
         */
        public static function findByPasswordResetToken($token)
        {
            if (!static::isPasswordResetTokenValid($token)) {
                return null;
            }
        
            return static::findOne(
                [
                    'password_reset_token' => $token,
                    'status'               => self::STATUS_ACTIVE,
                ]
            );
        }
    
        /**
         * Finds out if password reset token is valid
         *
         * @param string $token password reset token
         *
         * @return bool
         */
        public static function isPasswordResetTokenValid($token)
        {
            if (empty( $token )) {
                return false;
            }
        
            $timestamp = (int) substr($token, strrpos($token, '_') + 1);
            $expire = Yii::$app->params[ 'user.passwordResetTokenExpire' ];
            return $timestamp + $expire >= time();
        }
    
        /**
         * @inheritdoc
         */
        public function getId()
        {
            return $this->getPrimaryKey();
        }
    
        /**
         * @inheritdoc
         */
        public function getAuthKey()
        {
            return $this->auth_key;
        }
    
        /**
         * @inheritdoc
         */
        public function validateAuthKey($authKey)
        {
            return $this->getAuthKey() === $authKey;
        }
    
        /**
         * Validates password
         *
         * @param string $password password to validate
         *
         * @return bool if password provided is valid for current user
         */
        public function validatePassword($password)
        {
            return Yii::$app->security->validatePassword($password, $this->password_hash);
        }
    
        /**
         * Generates password hash from password and sets it to the model
         *
         * @param string $password
         */
        public function setPassword($password)
        {
            $this->password_hash = Yii::$app->security->generatePasswordHash($password);
        }
    
        /**
         * Generates "remember me" authentication key
         */
        public function generateAuthKey()
        {
            $this->auth_key = Yii::$app->security->generateRandomString();
        }
    
        /**
         * Generates new password reset token
         */
        public function generatePasswordResetToken()
        {
            $this->password_reset_token = Yii::$app->security->generateRandomString() . '_' . time();
        }
    
        /**
         * Removes password reset token
         */
        public function removePasswordResetToken()
        {
            $this->password_reset_token = null;
        }
        
        public function saveImage($file){
            /**
             * @var \yii\web\UploadedFile $file;
             */
            if (!empty($file)){
                if (!file_exists(\Yii::getAlias('@storage/author/') . $this->id)) {
                    FileHelper::createDirectory(\Yii::getAlias('@storage/author/') . $this->id);
                }
                if ($file->saveAs(\Yii::getAlias('@storage/author/').$this->id.'/'.$this->id.'.'.$file->extension)){
                    $this->avatar = $this->id.'.'.$file->extension;
                    return $this->save();
                }
                return false;
        
            }
            return true;
        }
        
    }
